package ru.ulstu.timeline.service;

import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import ru.ulstu.timeline.model.Event;
import ru.ulstu.timeline.model.EventDto;
import ru.ulstu.timeline.repository.EventRepository;
import ru.ulstu.user.model.User;
import ru.ulstu.user.model.UserDto;
import ru.ulstu.user.service.UserService;

import java.util.List;

import static ru.ulstu.core.util.StreamApiUtils.convert;

@Service
public class EventService {

    private final EventRepository eventRepository;
    private final UserService userService;

    public EventService(EventRepository eventRepository,
                        UserService userService) {
        this.eventRepository = eventRepository;
        this.userService = userService;
    }

    public List<EventDto> findAll() {
        return convert(eventRepository.findAll(), EventDto::new);
    }

    @Transactional
    public int create(EventDto eventDto) {
        return eventRepository.save(copyFromDto(new Event(), eventDto)).getId();
    }

    private Event copyFromDto(Event event, EventDto eventDto) {
        event.setCreateDate(eventDto.getCreateDate());
        event.setDescription(eventDto.getDescription());
        event.setRecipients(userService.findByIds(convert(eventDto.getRecipients(), UserDto::getId)));
        event.setTitle(eventDto.getTitle());
        event.setUpdateDate(eventDto.getUpdateDate());
        return event;
    }

    @Transactional
    public Integer update(EventDto eventDto) {
        Event event = eventRepository.findOne(eventDto.getId());
        return eventRepository.save(copyFromDto(event, eventDto)).getId();
    }

    @Transactional
    public void delete(Integer timelineId) {
        Event event = eventRepository.findOne(timelineId);
        eventRepository.delete(event);
    }

    public List<Event> findByIds(List<Integer> ids) {
        return eventRepository.findAll(ids);
    }
}
